// (C) Copyright Exoligent 2017.
// Authors and Contributors: Maxime Coroyer
// File containing device and software error codes

#ifndef _EXOERRORS_H_
#define _EXOERRORS_H_

#if defined(_WIN32) || defined(_WIN64)
#   include <windows.h>
#   if defined(EXOERRORS_STATIC)
#       define EXOERRORS_EXPORT
#   else
#       if defined(EXOERRORS_DLL_EXPORT)
#           define EXOERRORS_EXPORT __declspec(dllexport)
#       else
#           define EXOERRORS_EXPORT __declspec(dllimport)
#       endif
#   endif
#elif defined(__linux__) || defined(__APPLE__) || defined(__QNX__)
#   define EXOERRORS_EXPORT
#endif

#define MSG_SIZE    250
#define SUCCESS     0

// The error codes are 32-bit encoded and are defined as follows :
//  0x  F FF FF FFF
//      | |  |   |
//      | |  |   |___ Error index
//      | |  |_______ Sub-module index
//      | |__________ Module index
//      |____________ Provenance index
//
// Provenance index: Indicates the source of the error
// Note: Under Windows OS, the b1 bit is always set (Provenance index : xx1xb).
// Otherwise, the error returned is a Windows error system (cf. C bit - MSDN HResult)
// 0x2 : application (.exe)
// 0x3 : library (.so, .dll)
// 0x7 : protocol
// 0x6 : firmware
// Under GNU/Linux, there is no constraint on b1.
// 0x8 : kernel module (only linux)
//
//
// Module index: Indicates the product (or module) concerned.
// 0x2 : application (.exe)
//  0x00: FipLabs
//  0x01: FipWatcherServer
// 0x3 : library (.so, .dll)
//  0x00: FipCore
//  0x01: GenCard
// 0x4 : protocol
//  0x00: IProtocol
// 0x6 : firmware
//  0x00: FipCode
//
// Sub-module index: Indicates the file or class concerned
//
// Error index: Indicates the error number
//
// Example: 0x30100004
//  Provenance index:   0x3   -> library
//  Module index:       0x01  -> GenCard library
//  Sub-module index:   0x00  -> class: CGenCard
//  Error index:        0x004 -> error n°4: NO_REPLY_TO_READ_SN_REQUEST

//***************************************************************************************
//
//                                  ERROR CODES
//
//***************************************************************************************
//  FIRMWARE ERRORS
//***************************************************************************************
// --------------------------------------------------------------------------------------
//  FIPCODE
// --------------------------------------------------------------------------------------
#define ERROR_FIP_BASE                                      0x60000000
#define ERROR_FIP_CR_NON_SIGNIFIANT                         0x60000000
#define ERROR_FIP_CR_NON_PRODUCING                          0x60000001
#define ERROR_FIP_CR_NON_CONSUMING                          0x60000002
#define ERROR_FIP_CR_NON_COMPATIBLE                         0x60000003
#define ERROR_FIP_CR_FIFO_EVT_EMPTY                         0x60000004
#define ERROR_FIP_CR_FIFO_APER_FULL                         0x60000005
#define ERROR_FIP_CR_CMDE_INC                               0x60000006
#define ERROR_FIP_CR_OVERFLOW                               0x60000007
#define ERROR_FIP_CR_FIFO_MSG_EMI_INC                       0x60000008
#define ERROR_FIP_CR_FIFO_MSG_EMI_FULL                      0x60000009
#define ERROR_FIP_CR_FIFO_MSG_EMI_EMPTY                     0x6000000a
#define ERROR_FIP_CR_FIFO_MSG_REC_EMPTY                     0x6000000b
#define ERROR_FIP_CR_UNDERFLOW                              0x6000000c
#define ERROR_FIP_CR_DEF_CMDE_BA                            0x6000000d
#define ERROR_FIP_CR_DEF_ACQ_EOC                            0x6000000e

//***************************************************************************************
//  PROTOCOL ERRORS
//***************************************************************************************
// --------------------------------------------------------------------------------------
//  IPROTOCOL
// --------------------------------------------------------------------------------------
#define IP_ERROR_SW_WARNING                                 0x70000001
#define IP_ERROR_SW_READ_PARTIAL                            0x70000002
#define IP_ERROR_SW_FAIL                                    0x70000003
#define IP_ERROR_SW_EMPTY                                   0x70000004
#define IP_ERROR_SW_BAD_LENGTH                              0x70000005
#define IP_ERROR_SW_NOT_FOUND                               0x70000006
#define IP_ERROR_SW_BAD_REQUEST                             0x70000007
#define IP_ERROR_SW_BAD_CLA                                 0x70000008
#define IP_ERROR_SW_BAD_CTL                                 0x70000009
#define IP_ERROR_SW_BAD_P1                                  0x7000000A
#define IP_ERROR_SW_BAD_P2                                  0x7000000B
#define IP_ERROR_SW_DATA_TOO_SHORT                          0x7000000C
#define IP_ERROR_SW_DATA_TOO_LONG                           0x7000000D
#define IP_ERROR_SW_RQ_BAD_CRC                              0x7000000E
#define IP_ERROR_SW_CTL_NOT_SUPPORTED                       0x7000000F
#define IP_ERROR_SW_READ_ERROR                              0x70000010
#define IP_ERROR_SW_ID_NOT_EXIST                            0x70000011
#define IP_ERROR_SW_BAD_DST                                 0x70000012

//***************************************************************************************
//  ERREURS MODULE KERNEL - LINUX
//***************************************************************************************
#if defined(__linux__) || defined(__APPLE__)
#define ERROR_STATUS_TIMEOUT                                0x80000001
#define ERROR_STATUS_DEVICE_BUSY                            0x80000002
#define ERROR_STATUS_UNSUCCESSFUL                           0x80000003
#define ERROR_STATUS_INVALID_PARAMETER                      0x80000004
#define ERROR_STATUS_INVALID_DEVICE_STATE                   0x80000005
#endif

//***************************************************************************************
//  LIBRARY ERRORS
//***************************************************************************************
// --------------------------------------------------------------------------------------
//  FIPCORE DLL
// --------------------------------------------------------------------------------------
// --- ExoDeviceAccess
#define ERROR_CANNOT_LOCALLY_OPEN_DRIVER                    0x30000000
#define ERROR_INVALID_HND                                   0x30000001
#define ERROR_CANNOT_OPEN_DRIVER                            0x30000002
#define ERROR_CANNOT_CLOSE_DRIVER                           0x30000003
#define ERROR_CANNOT_CLOSE_TLS_DRIVER_HANDLE                0x30000004
#define ERROR_DRIVER_ALREADY_IN_USED                        0x30000005
#define ERROR_INVALID_MEM_ADDRESS_RANGE                     0x30000006
#define ERROR_INVALID_READ_RANGE                            0x30000007
#define ERROR_DATA_NOT_ENTIRELY_WRITTEN                     0x30000008
#define ERROR_INVALID_SUBSYSTEM_ID                          0x30000009
#define ERROR_INVALID_FIP_SPEED                             0x3000000A
#define ERROR_INVALID_FIP_IMPEDANCE                         0x3000000B

// --- KeyProtection
#define ERROR_SOFTWARE_PROTECTION                           0x30001000
#define ERROR_SOFTWARE_TYPE                                 0x30001001

// --- FipCodeInstall
#define ERROR_PERIODIC_CHANNEL_UNKNOWN                      0x30002000
#define ERROR_VARIABLE_LENGTH                               0x30002001
#define ERROR_REFRESH_TYPE_UNKNOWN                          0x30002002
#define ERROR_VARIABLE_TYPE_UNKNOWN                         0x30002003
#define ERROR_FULLFIP_ACCESS                                0x30002004
#define ERROR_WRONG_BA_FILE_FORMAT                          0x30002005
#define ERROR_UNKNOWN_BA_INSTRUCTION                        0x30002006
#define ERROR_CANNOT_OPEN_FIPCODE_FILE                      0x30002007
#define ERROR_FIPCODE_CHECK_FAILED                          0x30002008
#define ERROR_ILLEGAL_BA_PARAMETER                          0x30002009
#define ERROR_TOO_MANY_FRAME_DESCRIPTOR                     0x3000200A
#define ERROR_NOT_ENOUGH_MEMORY_FOR_CONFIGURATION           0x3000200B
#define ERROR_TOO_MANY_VARIABLE_DESCRIPTOR                  0x3000200C
#define ERROR_WRONG_FRAME_TYPE_IN_CONF_FILE                 0x3000200D
#define ERROR_NOT_ENOUGH_MEMORY_FOR_MSG_FIFO                0x3000200E
#define ERROR_NOT_ENOUGH_MEMORY_FOR_BA                      0x3000200F
#define ERROR_NOT_ENOUGH_MEMORY_FOR_BA1                     0x30002010
#define ERROR_NOT_ENOUGH_MEMORY_FOR_BA2                     0x30002011

// --- FipCodeAccess
#define ERROR_INVALID_FIP_USERCONF                          0x30003000
#define ERROR_INVALID_FIP_DEVICECONF                        0x30003001
#define ERROR_BA_PROG_NUMBER_UNKNOWN                        0x30003002

#define ERROR_INVALID_PAGE_NUMBER                           0x30004000
#define ERROR_PAGE_NOT_ENTIRELY_READ                        0x30004001
#define ERROR_PAGE_NOT_ENTIRELY_WRITE                       0x30004002
#define ERROR_INVALID_MEM_ADDRESS                           0x30004003
#define ERROR_INVALID_ACK_EOC                               0x30004004
#define ERROR_INVALID_CHANNEL_NUMBER                        0x30004005
#define ERROR_SIZE16_AND_VAR_LENGTH_NOT_EQUAL               0x30004006
#define ERROR_NOT_VAR_TIME                                  0x30004007
#define ERROR_SOFTWARE_NOT_OPERATIONAL                      0x30004008
#define ERROR_DATA_THREAD_STORAGE                           0x30004009

#define ERROR_NO_MEMORY_FOR_FIPCODEACCESS                   0x30005000
#define ERROR_NO_MEMORY_FOR_FIPCODECONF                     0x30005001
#define ERROR_NO_MEMORY_FOR_FIPUSERCONF                     0x30005002
#define ERROR_NO_MEMORY_FOR_FIPDEVICECONF                   0x30005003
#define ERROR_NO_MEMORY_FOR_FIPCONTEXT                     	0x30005004

#define ERROR_NO_MEMORY_FOR_VARIABLE_DESCRIPTOR_TAB         0x30005005
#define ERROR_NO_ACCESSKEY_FOUND                            0x30005006
#define ERROR_OPC_MEMORY_ALREADY_ALLOCATED                  0x30005007
#define ERROR_NO_MEMORY_FOR_OPC_SERVER                      0x30005008

// --- ParseConfFile
#define ERROR_IN_XML_PARSING                                0x30006000
#define ERROR_IN_XML_GET_ROOT_ELEMENT                       0x30006001
#define ERROR_IN_XML_GET_ELEMENT                            0x30006002
#define ERROR_INVALID_XML_ROOT_ELEMENT                      0x30006003
#define ERROR_INVALID_EXO_FILE_TYPE                         0x30006004

// --- FipCodeTLS
#define ERROR_IN_TLS_INIT                                   0x30007000

// --- FipCore
#define ERROR_NOT_IMPLEMENTED_FOR_THIS_DEVICE_TYPE          0x30008000
#define ERROR_IN_FIFO_WRITE_TEST                            0x30008001
#define ERROR_IN_FIFO_READ_TEST                             0x30008002
#define ERROR_IN_IRQ_TEST                                   0x30008003
#define ERROR_IN_REGISTERS_TEST                             0x30008004
#define ERROR_IN_TIMERS_TEST                                0x30008005
#define ERROR_INVALID_EVENT_DEFINITION                      0x30008006
#define ERROR_NULL_POINTER                                  0x30008007
#define ERROR_INVALID_ACCESSRIGHTS                          0x30008008
#define ERROR_CONF_NOT_FOUND                                0x30008009
#define ERROR_UNKNOWN_TAG_TYPE_ID                           0x3000800A
#define ERROR_FILE_ALREADY_ALLOCATED                        0x3000800B
#define ERROR_TRANSFER_TIMEOUT                              0x3000800C
#define ERROR_TRANSFER_ALREADY_IN_PROGRESS                  0x3000800D
#define ERROR_TRANSFER_RETRIES_EXCEED_LIMIT                 0x3000800E
#define ERROR_IN_RETRIEVING_FILE_NAME                       0x3000800F
#define ERROR_TRANSFER_ABORTED_BY_USER                      0x30008010
#define ERROR_FILE_RENAMING                                 0x30008011
#define ERROR_IN_FILE_READING                               0x30008012
#define ERROR_IN_FILE_WRITING                               0x30008013
#define ERROR_EOF_REACHED                                   0x30008014
#define ERROR_NOT_SUPPORT_FILE_TRANSFER 					0x30008015
#define ERROR_CHUNKS_LOST                                   0x30008016
// --- FipCore - OPC UA
#define ERROR_OPCUA_ALREADY_STARTED                         0x30008017
#define ERROR_OPCUA_ALREADY_STOPPED                         0x30008018
#define ERROR_OPCUA_RUN_STARTUP                             0x30008019

// --- Common
#define ERROR_IN_IOCTL_CALL                                 0x30009000
#define ERROR_INVALID_OUT_AFTER_IOCTL_CALL                  0x30009001

// --------------------------------------------------------------------------------------
//  GENCARD DLL
// --------------------------------------------------------------------------------------
// --- CGenCard
#define GENC_ERROR_NO_ABSTRACTCOM_ASSOCIATED                0x30100001
#define GENC_ERROR_NO_REPLY_TO_READ_MEMORY_REQUEST          0x30100002
#define GENC_ERROR_NO_REPLY_TO_READ_BY_ID_REQUEST           0x30100003
#define GENC_ERROR_NO_REPLY_TO_READ_SN_REQUEST              0x30100004
#define GENC_ERROR_NO_REPLY_TO_READ_LAST_SW_REQUEST         0x30100005
#define GENC_ERROR_NO_REPLY_TO_READ_ERROR_REQUEST           0x30100006
#define GENC_ERROR_NO_REPLY_TO_WRITE_MEMORY_REQUEST         0x30100007
#define GENC_ERROR_NO_REPLY_TO_WRITE_BY_ID_REQUEST          0x30100008
#define GENC_ERROR_NO_REPLY_TO_EXEC_BY_ID_REQUEST           0x30100009
#define GENC_ERROR_NO_REPLY_TO_RESET_REQUEST                0x3010000A
#define GENC_ERROR_NO_REPLY_TO_SET_MODE_REQUEST             0x3010000B
#define GENC_ERROR_NO_REPLY_TO_GET_MODE_REQUEST             0x3010000C
#define GENC_ERROR_READ_DATA_NUMBER_NOT_MATCH               0x3010000D
#define GENC_ERROR_WAIT_FOR_REPLY_TIMEOUT                   0x3010000E
#define GENC_ERROR_INCONSISTENT_FRAME                       0x3010000F
#define GENC_ERROR_WAIT_FOR_REPLY_MORE_THAN_ONE_FRAME       0x30100010
#define GENC_ERROR_NOT_IMPLEMENTED_FUNCTION                 0x30100011
#define GENC_ERROR_WAIT_FOR_EVENT_TIMEOUT                   0x30100012

// --- CAbstractCom
#define ABCOM_ERROR_COMMUNICATION_UNOPENED                  0x30101000

// --- CUsbFtd2xxCom
#define FTD2XX_ERROR_CANNOT_LOAD_FTD2XX                     0x30102000
#define FTD2XX_ERROR_CANNOT_READ_ALL_DATA_FROM_BUFFER       0x30102001
#define FTD2XX_ERROR_CANNOT_WRITE_ALL_DATA_TO_BUFFER        0x30102002
#define FTD2XX_ERROR_CANNOT_FIND_FTSETVIDPID                0x30102003
#define FTD2XX_ERROR_CANNOT_FIND_FTGETVIDPID                0x30102004
#define FTD2XX_ERROR_CANNOT_FIND_FTCREATEDEVICEINFOLIST     0x30102005
#define FTD2XX_ERROR_CANNOT_FIND_FTGETDEVICEINFOLIST        0x30102006
#define FTD2XX_ERROR_CANNOT_FIND_FTGETDEVICEINFODETAIL      0x30102007
#define FTD2XX_ERROR_CANNOT_FIND_FTLISTDEVICES              0x30102008
#define FTD2XX_ERROR_CANNOT_FIND_FTOPEN                     0x30102009
#define FTD2XX_ERROR_CANNOT_FIND_FTOPEN_EX                  0x3010200A
#define FTD2XX_ERROR_CANNOT_FIND_FTCLOSE                    0x3010200B
#define FTD2XX_ERROR_CANNOT_FIND_FTREAD                     0x3010200C
#define FTD2XX_ERROR_CANNOT_FIND_FTWRITE                    0x3010200D
#define FTD2XX_ERROR_CANNOT_FIND_FTSETBAUDRATE              0x3010200E
#define FTD2XX_ERROR_CANNOT_FIND_FTSETDIVISOR               0x3010200F
#define FTD2XX_ERROR_CANNOT_FIND_FTSETDATACHARACTERISTICS   0x30102010
#define FTD2XX_ERROR_CANNOT_FIND_FTSETTIMEOUTS              0x30102011
#define FTD2XX_ERROR_CANNOT_FIND_FTSETFLOWCONTROL           0x30102012
#define FTD2XX_ERROR_CANNOT_FIND_FTSETDTR                   0x30102013
#define FTD2XX_ERROR_CANNOT_FIND_FTCLRDTR                   0x30102014
#define FTD2XX_ERROR_CANNOT_FIND_FTSETRTS                   0x30102015
#define FTD2XX_ERROR_CANNOT_FIND_FTCLRRTS                   0x30102016
#define FTD2XX_ERROR_CANNOT_FIND_FTGETMODEMSTATUS           0x30102017
#define FTD2XX_ERROR_CANNOT_FIND_FTGETQUEUESTATUS           0x30102018
#define FTD2XX_ERROR_CANNOT_FIND_FTGETDEVICEINFO            0x30102019
#define FTD2XX_ERROR_CANNOT_FIND_FTGETDRIVERVERSION         0x3010201A
#define FTD2XX_ERROR_CANNOT_FIND_FTGETLIBRARYVERSION        0x3010201B
#define FTD2XX_ERROR_CANNOT_FIND_FTGETCOMPORTNUMBER         0x3010201C
#define FTD2XX_ERROR_CANNOT_FIND_FTGETSTATUS                0x3010201D
#define FTD2XX_ERROR_CANNOT_FIND_FTSETEVENTNOTIFICATION     0x3010201E
#define FTD2XX_ERROR_CANNOT_FIND_FTSETCHARS                 0x3010201F
#define FTD2XX_ERROR_CANNOT_FIND_FTSETBREAKON               0x30102020
#define FTD2XX_ERROR_CANNOT_FIND_FTSETBREAKOFF              0x30102021
#define FTD2XX_ERROR_CANNOT_FIND_FTPURGE                    0x30102022
#define FTD2XX_ERROR_CANNOT_FIND_FTRESETCOM                 0x30102023
#define FTD2XX_ERROR_CANNOT_FIND_FTRESETPORT                0x30102024
#define FTD2XX_ERROR_CANNOT_FIND_FTCYCLEPORT                0x30102025
#define FTD2XX_ERROR_CANNOT_FIND_FTRESCAN                   0x30102026
#define FTD2XX_ERROR_CANNOT_FIND_FTRELOAD                   0x30102027
#define FTD2XX_ERROR_CANNOT_FIND_FTSETRESETPIPERETRYCOUNT   0x30102028
#define FTD2XX_ERROR_CANNOT_FIND_FTSTOPINTASK               0x30102029
#define FTD2XX_ERROR_CANNOT_FIND_FTRESTARTINTASK            0x3010202A
#define FTD2XX_ERROR_CANNOT_FIND_FTSETDEADMANTIMEOUT        0x3010202B
#define FTD2XX_ERROR_CANNOT_FIND_FTREADEE                   0x3010202C
#define FTD2XX_ERROR_CANNOT_FIND_FTWRITEEE                  0x3010202D
#define FTD2XX_ERROR_CANNOT_FIND_FTERASEEE                  0x3010202E
#define FTD2XX_ERROR_CANNOT_FIND_FTEEREAD                   0x3010202F
#define FTD2XX_ERROR_CANNOT_FIND_FTEEREADEX                 0x30102030
#define FTD2XX_ERROR_CANNOT_FIND_FTEEPROGRAM                0x30102031
#define FTD2XX_ERROR_CANNOT_FIND_FTEEPROGRAMEX              0x30102032
#define FTD2XX_ERROR_CANNOT_FIND_FTEEUASIZE                 0x30102033
#define FTD2XX_ERROR_CANNOT_FIND_FTEEUAREAD                 0x30102034
#define FTD2XX_ERROR_CANNOT_FIND_FTEEUAWRITE                0x30102035
#define FTD2XX_ERROR_CANNOT_FIND_FTEEPROMREAD               0x30102036
#define FTD2XX_ERROR_CANNOT_FIND_FTEEPROMPROGRAM            0x30102037
#define FTD2XX_ERROR_CANNOT_FIND_FTSETLATENCYTIMER          0x30102038
#define FTD2XX_ERROR_CANNOT_FIND_FTGETLATENCYTIMER          0x30102039
#define FTD2XX_ERROR_CANNOT_FIND_FTSETBITMODE               0x3010203A
#define FTD2XX_ERROR_CANNOT_FIND_FTGETBITMODE               0x3010203B
#define FTD2XX_ERROR_CANNOT_FIND_FTSETUSBPARAMETERS         0x3010203C
#define FTD2XX_ERROR_INVALID_HANDLE                         0x3010203D
#define FTD2XX_ERROR_DEVICE_NOT_FOUND                       0x3010203E
#define FTD2XX_ERROR_DEVICE_NOT_OPENED                      0x3010203F
#define FTD2XX_ERROR_IO_ERROR                               0x30102040
#define FTD2XX_ERROR_INSUFFICIENT_RESOURCES                 0x30102041
#define FTD2XX_ERROR_INVALID_PARAMETER                      0x30102042
#define FTD2XX_ERROR_INVALID_BAUD_RATE                      0x30102043
#define FTD2XX_ERROR_DEVICE_NOT_OPENED_FOR_ERASE            0x30102044
#define FTD2XX_ERROR_DEVICE_NOT_OPENED_FOR_WRITE            0x30102045
#define FTD2XX_ERROR_FAILED_TO_WRITE_DEVICE                 0x30102046
#define FTD2XX_ERROR_EEPROM_READ_FAILED                     0x30102047
#define FTD2XX_ERROR_EEPROM_WRITE_FAILED                    0x30102048
#define FTD2XX_ERROR_EEPROM_ERASE_FAILED                    0x30102049
#define FTD2XX_ERROR_EEPROM_NOT_PRESENT                     0x3010204A
#define FTD2XX_ERROR_EEPROM_NOT_PROGRAMMED                  0x3010204B
#define FTD2XX_ERROR_INVALID_ARGS                           0x3010204C
#define FTD2XX_ERROR_NOT_SUPPORTED                          0x3010204D
#define FTD2XX_ERROR_OTHER_ERROR                            0x3010204E
#define FTD2XX_ERROR_DEVICE_LIST_NOT_READY                  0x3010204F

// --- CUsbCypCom
#define CYP_ERROR_INVALID_ENDPOINT                          0x30103000
#define CYP_ERROR_IN_ENDPOINT_TIMEOUT                       0x30103001
#define CYP_ERROR_OUT_ENDPOINT_TIMEOUT                      0x30103002
#define CYP_ERROR_DEVICE_NOT_OPENED                         0x30103003

// --------------------------------------------------------------------------------------
//  FIPBUSVIEW DLL
// --------------------------------------------------------------------------------------
// --- FipBusView
#define FBV_ERROR_NETWORK_NOT_STARTED                       0x30200001

//***************************************************************************************
//  APPLICATION ERRORS
//***************************************************************************************
// --------------------------------------------------------------------------------------
//  FIP LABS
// --------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------------
//  FIPWATCHER SERVER
// --------------------------------------------------------------------------------------
// --- FipWatcherCard
#define FWCARD_ERROR_INVALID_OBJECT                         0x20100000
#define FWCARD_ERROR_IS_ALREADY_LOCALLY_OPENED              0x20100001
#define FWCARD_ERROR_IS_ALREADY_REMOTELY_OPENED             0x20100002
#define FWCARD_ERROR_REMOTE_CONNECTION_TIMEOUT              0x20100003
#define FWCARD_ERROR_REMOTE_DISCONNECTION_TIMEOUT           0x20100004
#define FWCARD_ERROR_REMOTE_CONNECTION_INTERRUPTED          0x20100005
#define FWCARD_ERROR_INVALID_INPUT_PARAMETERS               0x20100006
#define FWCARD_ERROR_FCT_NOT_SUPPORTED_IN_REMOTE_CO         0x20100007
#define FWCARD_ERROR_DEVICE_NOT_OPENED                      0x20100008
#define FWCARD_ERROR_REPLY_SIZE                             0x20100009

#ifdef __cplusplus
    extern "C" {
#endif
// --------------------------------------------------------------------------------------
// Exportable functions
// --------------------------------------------------------------------------------------
EXOERRORS_EXPORT char* GET_EXO_ERROR(unsigned long error_code);

#ifdef __cplusplus
            }
#endif

#endif
