// (C) Copyright Exoligent 2017.
// Authors and Contributors: Maxime Coroyer
#ifndef _FIPCODECONSTANTS_H_
#define _FIPCODECONSTANTS_H_

#include "FipCode6/fipcode6.h"

// Defining constants internal to FipCodeAccess. Several types:
//  - Constants related to the FielDual component (for "Compatible mode")
//  - Constants related to the FullFIP component
//  - Constants related to the FipCode
//  - Constants related to our use of FipCodeAccess

//***************************************************************************************
//*                     Constants related to the FielDual component                     *
//***************************************************************************************
//**** REGISTERS
// Report Registers (read-only)
#define CHANNEL_STATUS          0x00
#define CLEAR_ERROR             0x04
#define RESET_CHANNEL1          0x08
#define RESET_CHANNEL2          0x0C
// Control registers (write only)
#define LOOPBACK_SELECTION      0x00
#define ENABLE_CHANNEL1         0x04
#define ENABLE_CHANNEL2         0x08
#define ENABLE_CHANNEL1_AND_2   0x0C

//**** REGISTER MASKS
// CHANNEL_STATUS
#define MSK_CHANNEL_STATUS_CHANNEL1_VALID       0x01    // Validity of channel 1
#define MSK_CHANNEL_STATUS_CHANNEL2_VALID       0x02    // Validity of channel 2
#define MSK_CHANNEL_STATUS_CHANNEL1_TX_ERROR    0x04    // Flag transmission error
                                                        // on channel 1
#define MSK_CHANNEL_STATUS_CHANNEL2_TX_ERROR    0x08    // Flag transmission error
                                                        // on channel 2
#define MSK_CHANNEL_STATUS_CHANNEL1_WATCHDOG    0x10    // Channel 1 watchdog reverse
                                                        // state flag
#define MSK_CHANNEL_STATUS_CHANNEL2_WATCHDOG    0x20    // Channel 2 watchdog reverse
                                                        // state flag
//***************************************************************************************
//*                     Constants related to the FullFIP component                      *
//***************************************************************************************
//**** LIMITATIVE CONSTANTS
#define BOARD_MEMORY_SIZE           0x800     // Number of embedded memory pages
#define NB_BYTE_PER_SEGMENT         128       // Number of 8-bits in one page
#define NB_WORD_PER_SEGMENT         64        // Number of 16-bits in one page
#define NB_DESCRIPTOR_PER_SEGMENT   4         // Number of descriptors in one page
#define MEMORY_ADDRESS_LIMIT        (BOARD_MEMORY_SIZE * NB_WORD_PER_SEGMENT)
#define BASE_MULTIPLICATOR          0x400     // Multiplier factor to obtain a word address 
                                              // for base0 and base1 (1kmots)
#define FIP_CONFIG_ID_MAX_SIZE      0x10000L  // Max size for FIP_CONFIG_ID zone: 64kwords
                                              // (frame descriptors)
#define FIP_CONFIG_VAR_MAX_SIZE     0x10000L  // Max size for FIP_CONFIG_VAR zone: 64kwords
                                              // (Variable descriptors
                                              // + FIFO Reception
                                              // + FIFO Emission)
#define FIP_CONFIG_BA_MAX_SIZE      0x10000L  // Max size for FIP_CONFIG_BA zone: 64kwords
                                              // (BA program)
#define FIP_DATA_USER_MAX_SIZE      0x80000L  // Max size for FIP_DATA_USER zone: 512kwords
                                              // (storage values for var. and msg)
#define FIP_CONFIG_BA_ANTIDUP_TP_OFFSET 0x080 // Offset of anti-duplication table for
                                              // TestP.
#define FIP_CONFIG_BA_FIFO_OFFSET   0x280     // Start Offset of FIFOs - FIP_CONFIG_BA
#define FIFO_RECEPTION_BLOCK_SIZE   4         // Size in words of 1 FIFO block
                                              // for message reception
#define FIFO_EMISSION_BLOCK_SIZE    5         // Size in words of 1 FIFO block
                                              // for message emission
#define FIFO_BA_BLOCK_SIZE          2         // Size in words of 1 FIFO block - BA

//**** REGISTERS [LA2 LA3 LA4 of the FullFip2 component ]
// Report registers (read-only)
#define USTATE      0x00 // A4A3A2 = 000
#define UFLAGS      0x04 // A4A3A2 = 001
#define VAR_STATE   0x08 // A4A3A2 = 010
#define SIZE16_L    0x0C // A4A3A2 = 011
#define SIZE16_H    0x10 // A4A3A2 = 100
// Control registers (write-only)
#define UCOM        0x00 // A4A3A2 = 000
#define USWITCH     0x04 // A4A3A2 = 001
#define KEY_H       0x08 // A4A3A2 = 010
#define KEY_L       0x0C // A4A3A2 = 011
#define TIME_H      0x10 // A4A3A2 = 100
#define TIME_L      0x14 // A4A3A2 = 101
// FIFO access register (read/write)
#define FILE_REG    0x1C // A4A3A2 = 111

//**** REGISTER MASKS
// USTATE
#define MSK_USTATE_IC       0x01    // IC:  Illegal order
#define MSK_USTATE_FE       0x02    // FE:  Format error
#define MSK_USTATE_AE       0x04    // AE:  Access error
#define MSK_USTATE_FR       0x08    // FR:  FIFO available in read / write
#define MSK_USTATE_SV       0x10    // SV:  Status validating read interpretation of VAR_STATE
                                    // and SIZE16 registers
#define MSK_USTATE_IRQ      0x20    // IRQ: Presence of IRQ channel external interruption
#define MSK_USTATE_EOC      0x40    // EOC: Presence of EOC channel external interruption
#define MSK_USTATE_BUSY     0x80    // BUSY: Transaction in progress
// UFLAGS
#define MSK_UFLAGS_STATUS               0x01
// VAR_STATE
#define MSK_VAR_STATE_CNF               0x80
//   - Var_State_Cr: Confirmation of executed primitive (0: ok, 1: not ok)
#define MSK_VAR_STATE_CR_ERROR_CODE     0x0F
//   - Var_State_VAR: Error code if confirmation is negative (CNF=1)
#define MSK_VAR_STATE_VAR_AP            0x20
#define MSK_VAR_STATE_VAR_ERROR_CODE    0x0F
//   - Var_State_MSG: Block number allocated in send FIFO (0..63) (message emission)
#define MSK_VAR_STATE_MSG_NUMBER        0x3F

//*** NETWORK_CONF MASKS
#define MSK_FDT_BEGIN_PTR   0x03ff  // Access mask to FDT_Begin_Ptr from FIP_Base_0

//**** ADDITIONAL MASKS
#define MSK_PROD_STATUS_MEANING         0x04
#define MSK_PROD_STATUS_REFRESH         0x01
#define MSK_PROD_STATUS_REFRESH_OK      0x05
                
//***************************************************************************************
//*                         Constants related to the FipCode                            *
//***************************************************************************************
// Constants related to the FIP_CONFIG_ID zone
#define ID_DESCRIPTOR_SIZE_IN_WORD  0x10L // Size of a frame descriptor (16-words)
#define ID_DESCRIPTOR_SIZE_IN_BYTE  0x20L // Size of a frame descriptor (32-bytes)
#define NB_ID_SM_MPS            4       // Number of SM_MPS frame descriptors
                                        // Network Management Frames
#define MAX_ID_NUMBER_LIMIT     4095L   // Max value for Max_ID_Number parameter
                                        // Note: Max_ID_Number defines the maximum number
                                        // of frame descriptors that the FullFIP can handle
                                        // [a reserved descriptor
                                        // + variable descriptors (SM_MPS + MPS)
                                        // + periodic message descriptors]
#define ID_NBR_PAR_LIMIT        0x0b    // Limit value of the ID_Nbr_Par parameter
#define PRODUCTION_PERIOD_LIMIT 0x3FFF  // Limit value for refreshing a variable
                                        // (in multiples of TSlot)
#define CONSUMPTION_PERIOD_LIMIT 0x3FFF // Limit value of the promptness of a variable
                                        // (in multiples of TSlot)

// Constants related to the FIP_CONFIG_VAR zone
#define VAR_DESCRIPTOR_SIZE_IN_WORD 0x10L // Size of a variable descriptor (16-words)
#define VAR_DESCRIPTOR_SIZE_IN_BYTE 0x20L // Size of a variable descriptor (32-bytes)
#define NB_VAR_SM_MPS           9       // Number of variable descriptors SM_MPS including
                                        // one reserved at the beginning of the table
                                        // [cf FIPCODE v6_4 p.222]
                                        // Network Management Frames
#define NB_FIFO_EMI             9       // Number of FIFOs for sending messages
                                        // FIFO 0: messages associated with the aperiodic channel
                                        // FIFO 1-8: messages associated with the periodic channel
#define NB_FIFO_REC             1       // Number of FIFOs for receiving messages
#define ID_9003                 0x9003  // Synchro var ID (Bus Arbiter)
#define PDU_LG_10xx             0x5029  // Pdu and length for "Identification" produced var
#define PDU_LG_P9003            0x5004  // Pdu and length for "Synchro BA" produced var 
#define PDU_LG_C9003            0x5006  // Pdu and length for "Synchro BA" consumed var
#define VAR_WITH_DYN_REF_MAX_SIZE   120 // Maximum length of userful data of a variable
                                        // with dynamic refreshment
#define VAR_WITH_REF_MAX_SIZE       125 // Maximum length of userful data of a variable
                                        // with refreshment
#define VAR_WITHOUT_REF_MAX_SIZE    126 // Maximum length of userful data of a variable
                                        // without refreshment 

#define MESSAGING_SEGMENT_TABLE_SIZE 256 // Size of the messaging segment configuration table

// Constants related to the FIP_CONFIG_BA zone
#define NB_FIFO_BA              3       // Number of FIFOs for the bus arbiter:
                                        // - aperiodic messages requests
                                        // - urgent aperiodic variable requests
                                        // - normal aperiodic variable requests
#define NB_ANTIDUP_TESTP_ELEMENT 256    // Number of elements in antiduplication table
                                        // TestP.
#define ANTIDUP_ELEMENT_SIZE_IN_WORD 2  // Size of antiduplication element. (2-words)
#define ANTIDUP_ELEMENT_SIZE_IN_BYTE 4  // Size of antiduplication element. (4-bytes)

// Constants related to the FIP_DATA_USER zone
#define MAX_MSG_LENGTH 266              // Maximum length of a message with header
                                        // + characteristics
#define MAX_MSG_LENGTH_WITHOUT_CHARACT  262 // Maximum length of a message with header
                                            // (without characteristics)
#define MAX_MSG_LENGTH_WITHOUT_CHARACT_AND_HEADER   256 // Maximum length of a message
                                                        // (without header and
                                                        // characteristics) 
#define MSG_LENGTH_HEADER       6       // Message header size in bytes

// Independent Area Constants
#define DESCRIPTOR_SIZE_IN_WORD 16      // Size of a descriptor (16-words)
#define DESCRIPTOR_SIZE_IN_BYTE 32      // Size of a descriptor (32-bytes)
#define NB_MAX_FIFO_EMISSION_BLOCK  64  // Maximum number of blocks for emission FIFO

#define FIP_NETWORK_STATUS_IN_BYTE  20  // Size of the network status table

#define BA_PARAMETERS_SIZE_IN_BYTE  10  // Parameter size of the arbiter editable "on-line"

#define POSITIVE_CNF            0       // Order report OK on Var_State_Var
#define NEGATIVE_CNF            1       // Order report NOK on Var_State_Var

#define RP_TIME_PAR_1M_MIN 45           // cf. Correction_Tps_Silence_Retournement.ods
#define RP_TIME_PAR_MAX 63
#define RP_TIME_PAR_1M_DEFAULT 45

#define SILENCE_PAR_1M_MIN 27           // cf. Correction_Tps_Silence_Retournement.ods
#define SILENCE_PAR_MAX 63
#define SILENCE_PAR_1M_DEFAULT 30

#define MAX_SEGMENT 256
#define MAX_STATION_ON_NETWORK 256
#define MAX_STATION_ON_SEGMENT 32
#define MAX_ID 65535
#define MAX_EVENT 255
#define MAX_APER_ID_LIST 64 

#define PRIORITY_LEVEL_MAX 15

#define TSLOT_MAX 3

// Constants related to the FIP frames
#define CTRL_ID_DAT         0x03
#define CTRL_ID_MSG         0x05
#define CTRL_ID_RQ1         0x29
#define CTRL_ID_RQ2         0x09
#define CTRL_RP_DAT         0x02
#define CTRL_RP_DAT_MSG     0x06
#define CTRL_RP_DAT_RQ1     0x2A
#define CTRL_RP_DAT_RQ2     0x0A
#define CTRL_RP_DAT_RQ1_MSG 0x2E
#define CTRL_RP_DAT_RQ2_MSG 0x0E
#define CTRL_RP_RQ1         0x28
#define CTRL_RP_RQ2         0x08
#define CTRL_RP_MSG_NOACK   0x04

#define CTRL_RP_MSG_ACKe    0x14
#define CTRL_RP_MSG_ACKo    0x94

#define CTRL_RP_ACKpe       0x30
#define CTRL_RP_ACKpo       0xB0
#define CTRL_RP_ACKme       0x10
#define CTRL_RP_ACKmo       0x90

#define CTRL_RP_FIN         0x40

//***************************************************************************************
//*                 Constants related to our use of FipCodeAccess                       *
//***************************************************************************************
// The following FipCode parameterization is used:
//  -   FullStation                         : <FIP_Type_Station>    = STATION_4
//  -   EOC external interruption           : <FIP_EOC_PULSE>       = EOC_PULSE
//  -   Classic Memory Access               : <FIP_Mode_Fonct>      = OPTION_1 
//  -   Bimedium card                       : <FIP_Redundancy>      = RDD_USER
//  -   Internal transmission clock         : <FIP_Net_Cnt>         = MODE_TXCK
//  => These parameters are encoded directly in FipCodeAccess.c

//  -   Number of repeat sending FIP messages awaiting acknowledgment:
//      <MSG_Repeat> = 3
#define NB_OF_MSG_REPEAT    0x3

//  -   Size of the FIFOs (emission of messages,
//                        réception of messages, 
//                        management of the bus arbiter) : 120 mots
//       + A size common to all FIFOs is chosen.
//       + 120 is divisible by the block size of each type of FIFO
//  This gives the following relations:
//  -> size of a block for an emission FIFO (FIFO_EMISSION_BLOCK_SIZE) : 5
//      => number of blocks = 120 / 5 = 24 => 24 msg / emission FIFO
//  -> size of a block for a reception FIFO (FIFO_RECEPTION_BLOCK_SIZE) : 4
//      => number of blocks = 120 / 4 = 30 => 30 msg / reception FIFO
//  -> size of a block for FIFO BA (FIFO_BA_BLOCK_SIZE) : 2
//      => number of blocks = 120 / 2 = 60
#define FIFO_SIZE           0x0078
#define NB_FIFO_RECEPTION_BLOCK (FIFO_SIZE / FIFO_RECEPTION_BLOCK_SIZE)
#define NB_FIFO_EMISSION_BLOCK  (FIFO_SIZE / FIFO_EMISSION_BLOCK_SIZE)
#define NB_FIFO_BA_BLOCK        (FIFO_SIZE / FIFO_BA_BLOCK_SIZE) 

// Chaînes de caractères utiles
#define SPEED_31K25_STR "31.25 kbits/s"
#define SPEED_1M_STR    "1 Mbits/s"
#define SPEED_2M5_STR   "2.5 Mbits/s"
#define SPEED_5M_STR    "5 Mbits/s"

#define FRAME_TYPE_FIP_STR      "FIP"
#define FRAME_TYPE_WORLDFIP_STR "WorldFIP"
#define FRAME_TYPE_SLOWFIP_STR  "SlowFIP"

#define PDU_MPS_STR     "MPS"
#define PDU_SM_MPS_STR  "SM_MPS"

#define OPTIMIZATION0_STR "Optimization 0"
#define OPTIMIZATION1_STR "Optimization 1"
#define OPTIMIZATION2_STR "Optimization 2"
#define OPTIMIZATION3_STR "Optimization 3"

#define SEND_ID_DAT_STR "SEND_ID_DAT"
#define SEND_ID_MSG_STR "SEND_ID_MSG"
#define SEND_MSG_STR    "SEND_MSG"
#define SEND_APER_STR   "SEND_APER"
#define NEXT_MACRO_STR  "NEXT_MACRO"
#define SUSPEND_STR     "SUSPEND"
#define WAIT_TIME_STR   "WAIT_TIME"
#define WAIT_SYNC_STR   "WAIT_SYNC"
#define TESTP_STR       "TESTP"

#define MSG_REC_CH0_STR "Aperiodic ch.0"
#define MSG_REC_CH1_STR "Periodic ch.1"
#define MSG_REC_CH2_STR "Periodic ch.2"
#define MSG_REC_CH3_STR "Periodic ch.3"
#define MSG_REC_CH4_STR "Periodic ch.4"
#define MSG_REC_CH5_STR "Periodic ch.5"
#define MSG_REC_CH6_STR "Periodic ch.6"
#define MSG_REC_CH7_STR "Periodic ch.7"
#define MSG_REC_CH8_STR "Periodic ch.8"

#define MSG_APERIODIC_MODE_STR  "Aperiodic"
#define MSG_PERIODIC_MODE_STR   "Periodic"

#define DIRECTION_IN_STR "IN"
#define DIRECTION_OUT_STR "OUT"

#define MODE_URGENT_STR "Urgent"
#define MODE_NORMAL_STR "Normal"

#define BIG_ENDIAN_STR "Big-Endian"
#define LITTLE_ENDIAN_STR "Little-Endian"

#define MAX_PATH_LENGTH 260
#define MAX_NAME_LENGTH 255
#define MAX_TYPE_LENGTH 255
#define MAX_UNIT_LENGTH 255

// Tags Constants 
#define TAG_TYPES_BOOLEAN   0
#define TAG_TYPES_INT8      1
#define TAG_TYPES_UINT8     2
#define TAG_TYPES_INT16     3
#define TAG_TYPES_UINT16    4
#define TAG_TYPES_INT32     5
#define TAG_TYPES_UINT32    6
#define TAG_TYPES_INT64     7
#define TAG_TYPES_UINT64    8
#define TAG_TYPES_FLOAT     9
#define TAG_TYPES_DOUBLE    10
#define TAG_TYPES_ASCII     11
#define TAG_TYPES_HEXA      14

// File Transfer
#define FILE_RECEPTION_TIMEOUT 5 //s
#define FILE_RETRIES_NUMBER 10
#define FILE_CHUNK_SIZE 250




#endif
